package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.entity.User;
import com.example.nto.service.EmployeeService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;
//я поменял на BAD_REQUEST 06.12.24 23:00

@RestController
@RequestMapping("/api")
public class EmployeeController {
    private final EmployeeService employeeService;

    public EmployeeController(EmployeeService employeeService) {
        this.employeeService = employeeService;
    }

    @PreAuthorize("hasRole('ADMIN')")
    @GetMapping("/admin/employees")
    public ResponseEntity<List<Employee>> getAllEmployees() {
        List<Employee> employees = employeeService.findAll();
        return ResponseEntity.ok(employees);
    }



    @GetMapping("/{login}/auth")//auth
    public ResponseEntity<?> authenticate(@PathVariable String login) {
        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isPresent()) {
            return ResponseEntity.ok("Valid login");


        } else {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid login");

        }

    }

    @GetMapping("/{login}/info")
    public ResponseEntity<?> getInfo(@PathVariable String login) {
        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isPresent()) {
            return ResponseEntity.ok(employee.get());
        }
        return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid login");
    }


    @PatchMapping("/{login}/open")//open
    public ResponseEntity<?> openDoor(@PathVariable String login, @RequestBody Map<String, Long> payload) {
        Long code = payload.get("value");
        if (code == null) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid payload");

        }



        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isEmpty()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid login");

        }



        if (employeeService.validateCode(login, code)) {
            return ResponseEntity.ok("Door opened");

        } else {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid code");
        }
    }

    @GetMapping("/auth")
    public ResponseEntity<?> authenticate(@RequestParam String username, @RequestParam String password) {
        User user = userService.findByUsername(username);
        if (user != null && passwordEncoder.matches(password, user.getPassword())) {
            return ResponseEntity.ok("Valid login");
        }
        return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid login");
    }
}
// made by truettwo and maks ))